* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* THIS FILE CONTAINS A SERIES OF FUNCTIONS WHICH CAN BE USED TO     *
* PRINT CODE 3 OF 9 BARCODES USING ANY LASER PRINTER WHICH CONFORMS *
* TO PCL-5 CONVENTIONS.  NO SPECIAL FONT CARTRIDGE OR DRIVERS ARE   *
* REQURIRED                                                         *
*                                                                   *
* These functions have been extensively tested on the HP Laserjet 4 *
* and work perfectly.  I've also used them with a LAN-based fax     *
* modem that uses PCL5.  You may need to make some minor adjustments*
* if you're using a different printer or device.                    *
*                                                                   *
* Several Functions are included.  They are used as follows:        *
*                                                                   *
* STR2BAR( char )                                                   *
*   This function creates a binary representation of what the       *
*   barcode will look like when printed, using 0 for spaces and     *
*   1 for lines.  For example, a barcode which looks like...        *
*          Converts to "10010101100110101"                 *
*          (The character shown is the 'START' character)  *
*                                                                   *
* BAR2PCL5( char, int, int, int, int )                              *
*   This function takes the binary file created by STR2BAR() and    *
*   converts it to printer control codes, printing the barcode at   *
*   the designated position on the page.  This command does not     *
*   effect the value of _PLINENO or _PCOLNO, and does not change    *
*   the position of the cursor after the barcode has been printed.  *
*   Your routine must allow for space to be left blank on the page  *
*   for the barcode.                                                *
*                                                                   *
* PCLVLINE( int, int )                                              *
*   Returns the printer control codes to draw a verticle line       *
*                                                                   *
* PCLLOC( int, int )                                                *
*   Returns the printer control codes to position the printer at    *
*   a specified X,Y coordinate                                      *
*                                                                   *
* COL2PCL5( int )                                                   *
*   Converts standard 10 pitch column numbers to the PCL5           *
*   coordinates used by the other commands.  This conversion seems  *
*   to be slightly different for different printers, and for        *
*   different modes.  I tested it on an HP Laserjet 4 in portriat   *
*   mode, using 10 pitch Courier                                    *
*                                                                   *
* ROW2PCL5( int )                                                   *
*   Converts standard 10 pitch row numbers to the PCL5              *
*   coordinates used by the other commands.  See the note in        *
*   COL2PCL5()                                                      *
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* COMMENTS WELCOME TO DAVE CASS                                     *
*                                                                   *
*      Mail:    Commanding Officer                                  *
*               Fleet and Industrial Supply Center                  *
*               Attn Code 302.32                                    *
*               1968 Gilbert St Ste 600                             *
*               Norfolk VA  23511-3396                              *
*                                                                   *
*      Fax:     (804)444-3087                                       *
*      E-mail:  LTJGCASS@AOL.COM                                    *
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

* SAMPLE *

SET PRINTER TO \\SPOOLER\Q=LASER_FRONT\NB
SET PRINT ON

* Print a barcode of the word "TEST" at the PCL5 coordinates 100,100
* the barcode is 300 dots high, and each line is 15 dots wide
=BAR2PCL5( STR2BAR("TEST"), 100, 100, 300, 15 )

* Print another barcode, this one the word "TEST2" at the row/column
* 25,25   This barcode is the same size as the first.
=BAR2PCL5( STR2BAR("TEST2"), ROW2PCL5( 25 ), COL2PCL5( 25 ), 300, 15 )

EJECT
SET PRINT OFF
SET PRINTER TO

* END OF SAMPLE *



FUNCTION STR2BAR
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   *   Function to convert a string of text into a binary  *
   *   indication of the Code 3 of 9 characters.           *
   *                                                       *
   *   Author:  David Cass                                 *
   *   Date:    21 DEC 94                                  *
   *                                                       *
   *   Parameters:  mpc_string    String to be encoded     *
   *                                                       *
   *   Returns:     char          String of 1's and 0's    *
   *                              representing bars and    *
   *                              spaces in the 3 of 9     *
   *                              code                     *
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   PARAMETERS mpc_string
   PRIVATE ALL
   STORE "{"+UPPER(mpc_string)+"}" TO mpc_string
   STORE "" TO mc_return
   FOR mi_cnt=1 TO LEN(mpc_string)
      DO CASE
         CASE SUBSTR(mpc_string,mi_cnt,1)="{"
            STORE mc_return + "1001011011010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="}"
            STORE mc_return + "1001011011010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="A"
            STORE mc_return + "1101010010110" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="B"
            STORE mc_return + "1011010010110" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="C"
            STORE mc_return + "1101101001010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="D"
            STORE mc_return + "1010110010110" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="E"
            STORE mc_return + "1101011001010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="F"
            STORE mc_return + "1011011001010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="G"
            STORE mc_return + "1010100110110" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="H"
            STORE mc_return + "1101010011010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="I"
            STORE mc_return + "1011010011010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="J"
            STORE mc_return + "1010110011010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="K"
            STORE mc_return + "1101010100110" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="L"
            STORE mc_return + "1011010100110" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="M"
            STORE mc_return + "1101101010010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="N"
            STORE mc_return + "1010110100110" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="O"
            STORE mc_return + "1101011010010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="P"
            STORE mc_return + "1011011010010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="Q"
            STORE mc_return + "1010101100110" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="R"
            STORE mc_return + "1101010110010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="S"
            STORE mc_return + "1011010110010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="T"
            STORE mc_return + "1010110110010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="U"
            STORE mc_return + "1100101010110" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="V"
            STORE mc_return + "1001101010110" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="W"
            STORE mc_return + "1100110101010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="X"
            STORE mc_return + "1001011010110" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="Y"
            STORE mc_return + "1100101101010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="Z"
            STORE mc_return + "1001101101010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="0"
            STORE mc_return + "1010011011010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="1"
            STORE mc_return + "1101001010110" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="2"
            STORE mc_return + "1011001010110" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="3"
            STORE mc_return + "1101100101010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="4"
            STORE mc_return + "1010011010110" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="5"
            STORE mc_return + "1101001101010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="6"
            STORE mc_return + "1011001101010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="7"
            STORE mc_return + "1010010110110" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="8"
            STORE mc_return + "1101001011010" TO mc_return
         CASE SUBSTR(mpc_string,mi_cnt,1)="9"
            STORE mc_return + "1011001011010" TO mc_return
      ENDCASE
   ENDFOR
   RETURN mc_return

FUNCTION BAR2PCL5
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   *   Function to move through the converted string       *
   *   one character at a time, and print a bar if the     *
   *   character is '1' or a space if it is '0'.           *
   *                                                       *
   *   Author:  David Cass                                 *
   *   Date:    21 DEC 94                                  *
   *                                                       *
   *   Parameters:  mpc_pattern   A string of 0's and 1's  *
   *                              representing the pattern *
   *                              of the barcode to print  *
   *                                                       *
   *                mpi_yc        Position of top-left     *
   *                              corner of code - Y Axis  *
   *                                                       *
   *                mpi_xc        Position of top-left     *
   *                              corner of code - X Axis  *
   *                                                       *
   *                mpi_h         Height of the bar        *
   *                                                       *
   *                mpi_w         Width each bar component *
   *                                                       *
   *   Returns:     none                                   *
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   PARAMETER mpc_pattern, mpi_yc, mpi_xc, mpi_h, mpi_w
   PRIVATE ALL
   STORE SET("CONSOLE") TO mc_console
   STORE SET("TALK") TO mc_talk
   STORE _PLINENO TO mi_pline
   STORE _PCOLNO TO mi_pcolno
   SET TALK OFF
   SET CONSOLE OFF
   ?? CHR(27)+CHR(38)+CHR(102)+CHR(48)+CHR(83)
   FOR mi_count = 1 TO LEN(mpc_pattern)
      IF SUBSTR(mpc_pattern,mi_count,1)="1"
         ?? PCLLOC(mpi_yc,mpi_xc)+PCLVLINE(mpi_h,mpi_w)
      ENDIF
      STORE mpi_xc+mpi_w TO mpi_xc
   ENDFOR
   ?? CHR(27)+CHR(38)+CHR(102)+CHR(49)+CHR(83)
   IF mc_console == "ON"
      SET CONSOLE ON
   ENDIF
   IF mc_talk == "ON"
      SET TALK ON
   ENDIF
   STORE mi_pline TO _PLINENO
   STORE mi_pcolno TO _PCOLNO
   RETURN

FUNCTION PCLVLINE
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   *   Function to move through the converted string       *
   *   one character at a time, and print a bar if the     *
   *   character is '1' or a space if it is '0'.           *
   *                                                       *
   *   Author:  David Cass                                 *
   *   Date:    21 DEC 94                                  *
   *                                                       *
   *                mpi_len       The length, in printer   *
   *                              dots, of the line        *
   *                                                       *
   *                mpi_thick     The thickness, in        *
   *                              printer dots, of the line*
   *                                                       *
   *   Returns:     char string   PCL5 code for the line   *
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   PARAMETER mpi_len,mpi_thick
   PRIVATE ALL
   STORE ALLTRIM(STR(mpi_len)) TO mpi_len
   STORE ALLTRIM(STR(mpi_thick)) TO mpi_thick
   RETURN CHR(27)+"*c"+mpi_len+"V"+CHR(27)+"*c"+mpi_thick+"H"+CHR(27)+"*c0P"
   
FUNCTION PCLLOC
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   *   Function to move through the converted string       *
   *   one character at a time, and print a bar if the     *
   *   character is '1' or a space if it is '0'.           *
   *                                                       *
   *   Author:  David Cass                                 *
   *   Date:    21 DEC 94                                  *
   *                                                       *
   *   Parameters:  mpi_yc        Position of top-left     *
   *                              corner - Y Axis          *
   *                                                       *
   *                mpi_xc        Position of top-left     *
   *                              corner - X Axis          *
   *                                                       *
   *   Returns:     char string   PCL5 code for move       *
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   PARAMETER mpi_yc,mpi_xc
   PRIVATE ALL
   STORE ALLTRIM(STR(mpi_yc)) TO mpi_yc
   STORE ALLTRIM(STR(mpi_xc)) TO mpi_xc
   RETURN CHR(27)+"&a"+mpi_yc+"V"+CHR(27)+"&a"+mpi_xc+"H"

FUNCTION COL2PCL5
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   *   Function to convert a printer PCOL() position to    *
   *   a PCL5() coordinate.  (Works only for 10-pitch      *
   *   fonts)                                              *
   *                                                       *
   *   Author:  David Cass                                 *
   *   Date:    21 DEC 94                                  *
   *                                                       *
   *   Parameters:  mpi_col       PCOL() position to be    *
   *                              converted                *
   *                                                       *
   *   Returns:     int           PCL5 coordinate          *
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   PARAMETER mpi_col
   PRIVATE ALL
   RETURN (mpi_col*72)-60
   
FUNCTION ROW2PCL5   
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   *   Function to convert a printer PROW() position to    *
   *   a PCL5() coordinate.  (Works only for 10-pitch      *
   *   fonts)                                              *
   *                                                       *
   *   Author:  David Cass                                 *
   *   Date:    21 DEC 94                                  *
   *                                                       *
   *   Parameters:  mpi_row       PROW() position to be    *
   *                              converted                *
   *                                                       *
   *   Returns:     int           PCL5 coordinate          *
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   PARAMETER mpi_row
   PRIVATE ALL
   RETURN (mpi_row*109)+10
